#include "EasyVR.h"
#include "main.h"
#include "stm32f0xx.h"
#include "uart.h"
#include "protocol.h"

#define true 1
#define false 0

struct {
	
	uint8_t _value;
	uint8_t v;
	
	struct {
			uint8_t _command : 1;
      uint8_t _builtin : 1;
      uint8_t _error : 1;
      uint8_t _timeout : 1;
      uint8_t _invalid : 1;
      uint8_t _memfull : 1;
      uint8_t _conflict : 1;
      uint8_t _token : 1;
	} b;
	
} _status;

/*********************************************************/

void EVRSend (uint8_t c) {
	
	delay_ms(1);
	uartSendInt(&c, EVRUsart);
	}

void EVRSendCmd (uint8_t c) {
	
	uartSendInt(&c, EVRUsart);
	}

void EVRSendArg(int8_t c) {
	
	EVRSend(c + ARG_ZERO);
	}

void EVRSendGroup(uint8_t c) {
	
	c = c + ARG_ZERO;
	delay_ms(1);
	uartSendInt(&c, EVRUsart);
	delay_ms(19);	 // worst case time to cache a full group in memory
	}

int EVRRecv(int16_t timeout) { //negative timeout means forever
	//vidjeti samo kada timeout dosegne 0, jel ceka RXNE zastavicu
	while(timeout != 0 && (USART_GetFlagStatus(EVRUsart, USART_FLAG_RXNE) == RESET)) {
		
		delay_ms(1);
		if ( timeout > 0)
			--timeout;
	}
	//ovisno o gornjem komentaru odrediti koji od ova 2 ide
	return USART_ReceiveData(EVRUsart);
	//return uartGetc(EVRUsart);
	}

bool EVRRecvArg(int8_t *c, int16_t timeout) {
	
	int r;
	EVRSend(ARG_ACK);
	r = EVRRecv(timeout);
	*c = r - ARG_ZERO;
	return (r>=ARG_MIN && r<=ARG_MAX);
	}

/******************************************************************/

bool EVRdetect (void) {
	
	uint8_t i;
	for(i=0; i<5; ++i) {
		
		EVRSendCmd(CMD_BREAK);
		if ( EVRRecv(WAKE_TIMEOUT) == STS_SUCCESS)
			return true;
		}
	return false;
	}

bool EVRstop(void) {
	
	uint8_t rx;
	EVRSendCmd(CMD_BREAK);
	
	rx = EVRRecv(WAKE_TIMEOUT);
	if (rx == STS_INTERR || rx == STS_SUCCESS)
		return true;
	return false;
}

bool EVRsetLanguage(int8_t lang)
{        
  EVRSendCmd(CMD_LANGUAGE);
  EVRSendArg(lang);

  if (EVRRecv(DEF_TIMEOUT) == STS_SUCCESS)
    return true;
  return false;
}

bool EVRsetTimeout(int8_t seconds)
{
  EVRSendCmd(CMD_TIMEOUT);
  EVRSendArg(seconds);

  if (EVRRecv(DEF_TIMEOUT) == STS_SUCCESS)
    return true;
  return false;
}

//Pisalo uint8_t& training
bool EVRdumpCommand(int8_t group, int8_t index, char* name, uint8_t *training)
{
	int8_t rx, len;
  EVRSendCmd(CMD_DUMP_SD);
  EVRSendGroup(group);
  EVRSendArg(index);

  if (EVRRecv(DEF_TIMEOUT) != STS_DATA)
    return false;
  
  
  if (!EVRRecvArg(&rx, DEF_TIMEOUT))
    return false;
  *training = rx & 0x07;
  if (rx == -1 || *training == 7)
    training = 0;
  
  _status.v = 0;
  _status.b._conflict = (rx & 0x18) != 0;
  _status.b._command = (rx & 0x08) != 0;
  _status.b._builtin = (rx & 0x10) != 0;
  
  if (!EVRRecvArg(&rx, DEF_TIMEOUT))
    return false;
  _status._value = rx;

  if (!EVRRecvArg(&rx, DEF_TIMEOUT))
    return false;
  len = rx == -1 ? 32 : rx;
  for ( ; len > 0; --len, ++name)
  {
    if (!EVRRecvArg(&rx, DEF_TIMEOUT))
      return false;
    if (rx == '^' - ARG_ZERO)
    {
      if (!EVRRecvArg(&rx, DEF_TIMEOUT))
        return false;
      *name = '0' + rx;
      --len;
    }
    else
    {
      *name = ARG_ZERO + rx;
    }
  }
  *name = 0;
  return true;
}

void EVRrecognizeCommand(int8_t group)
{
  EVRSendCmd(CMD_RECOG_SD);
  EVRSendArg(group);
}

void EVRrecognizeWord(int8_t wordset)
{
  EVRSendCmd(CMD_RECOG_SI);
  EVRSendArg(wordset);
}

bool EVRhasFinished(void)
{
  int8_t rx = EVRRecv(NO_TIMEOUT);
  if (rx < 0)
    return false;
  
  _status.v = 0;
  
  switch (rx)
  {
  case STS_SUCCESS:
    return true;
  
  case STS_SIMILAR:
    _status.b._builtin = true;
    goto GET_WORD_INDEX;

  case STS_RESULT:
    _status.b._command = true;
  
  GET_WORD_INDEX:
    if (EVRRecvArg(&rx, DEF_TIMEOUT))
    {
      _status._value = rx;
      return true;
    }
    break;
    
  case STS_TOKEN:
    _status.b._token = true;
  
    if (EVRRecvArg(&rx, DEF_TIMEOUT))
    {
      _status._value = rx << 5;
      if (EVRRecvArg(&rx, DEF_TIMEOUT))
      {
        _status._value |= rx;
        return true;
      }
    }
    break;
    
  case STS_TIMEOUT:
    _status.b._timeout = true;
    return true;
    
  case STS_INVALID:
    _status.b._invalid = true;
    return true;
    
  case STS_ERROR:
    _status.b._error = true;
    if (EVRRecvArg(&rx, DEF_TIMEOUT))
    {
      _status._value = rx << 4;
      if (EVRRecvArg(&rx, DEF_TIMEOUT))
      {
        _status._value |= rx;
        return true;
      }
    }
    break;
  }

  // unexpected condition (communication error)
  _status.v = 0;
  _status.b._error = true;
  return true;
}

int8_t EVRgetCommand(void) { 

	if(_status.b._command)
		return (_status._value);
	else
		return -1;
}

int8_t EVRgetWord(void) { 

	if(_status.b._builtin)
		return (_status._value);
	else
		return -1;

}

int16_t EVRgetError(void) { 
	
	if(_status.b._error)
		return (_status._value);
	else
		return -1;
}

bool EVRisTimeout(void) { 
	return _status.b._timeout; 
}

bool EVRsetPinOutput(int8_t pin, int8_t value)
{
  EVRSendCmd(CMD_QUERY_IO);
  EVRSendArg(pin);
  EVRSendArg(value);

  if (EVRRecv(DEF_TIMEOUT) == STS_SUCCESS)
    return true;
  return false;
}

bool EVRplaySound(int16_t index, int8_t volume)
{
  EVRSendCmd(CMD_PLAY_SX);
  EVRSendArg((index >> 5) & 0x1F);
  EVRSendArg(index & 0x1F);
  EVRSendArg(volume);

  if (EVRRecv(PLAY_TIMEOUT) == STS_SUCCESS)
    return true;
  return false;
}

/*
void EVRplaySoundAsync(int16_t index, int8_t volume)
{
  EVRSendCmd(CMD_PLAY_SX);
  EVRSendArg((index >> 5) & 0x1F);
  EVRSendArg(index & 0x1F);
  EVRSendArg(volume);
}
*/
