#include "main.h"
#include "stm32f0xx_gpio.h"
#include "uart.h"
#include "EasyVR.h"
#include "stm32f0xx.h"
#include "initFile.h"
#include "nRF24L01.h"
#include "spi.h"
#include <stdio.h>

void EVRaction(void);

static uint32_t TimingDelay;

//Delay u destcima mikrosekundi delay_ms(1) = 1ms
void delay_ms(uint32_t nTime)
{ 
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

void TimingDelay_Decrement(void)
{
	if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

//Groups and Commands
enum Groups
{
  GROUP_0  = 0,
  GROUP_1  = 1,
	GROUP_2  = 2,
};

enum Group0 
{
  G0_POKRENI = 0,
};

enum Group1 
{
  G1_GLAZBA = 0,
  G1_SVJETLO = 1,
};

enum Group2 
{
  G2_UPALI = 0,
  G2_UGASI = 1,
};

uint8_t group;
int8_t idx;
int16_t err;
unsigned char sendAction;

int main (void) {
	
	//Postavi SysTick da broji po 1ms, racuna se prema formuli:
	//Reload Value = SysTick Counter Clock (Hz) x  Desired Time base (s)
	if (SysTick_Config(SystemCoreClock / 1000))
  { 
    /* Capture error */ 
    while (1);
  }
	NVIC_SetPriority(SysTick_IRQn, 0);
	
	initBlueLED();
	spi_nrfInit();
	
	uartEVROpen(USART1, 9600); //PA9 i PA10
	uartPCOpen(USART2, 9600);  //PA2 i PA3
	
	if(!EVRdetect()) {
		
		uartPutString("EasyVR not detected! ", USART2);
		while(1); //Error loop
		}
	
	EVRsetPinOutput(IO1, RESET);
	uartPutString("EasyVR detected! ", USART2);
	EVRsetTimeout(5);
	EVRsetLanguage(ENGLISH);
	
	group = TRIGGER;
	
	while(1) {
		
		EVRsetPinOutput(IO1, SET); //LED on (listening)
		uartPutString("Say command in Group ", USART2);
		uartSendInt(&group, USART2);
		EVRrecognizeCommand(group);
		
		do {
			//tu se moze nesto raditi dok se ceka naredba
			delay_ms(3000);
		} while (!EVRhasFinished());
		
		EVRsetPinOutput(IO1, RESET); //LED off
		
		idx = EVRgetWord();
		if (idx >=0) {
			
			group = GROUP_1;
		}
		idx = EVRgetCommand();
		if (idx >= 0) {
			uint8_t train = 0;
			char name[32];
			uartPutString("Command: ", USART2);
			uartSendInt(&idx, USART2);
			if(EVRdumpCommand(group, idx, &name[0], &train)) {
				
				uartPutString(" = ", USART2);
				uartPutString(name, USART2);
			}
			else
				uartPutString("nesto ", USART2);
				
			EVRplaySound(BEEP, VOL_FULL);
			//some action
			EVRaction();
		}
		else {  //errors or timeout
			if(EVRisTimeout()){
				uartPutString("Timed out, try again...", USART2);
			}
			err = EVRgetError();
			if (err >= 0) {
				uartPutString("Error ", USART2);
				//uartPutString(err, USART2);
			}
		}
		
	}
}

void EVRaction (void) {
	switch (group) {
		
		case GROUP_0:
			switch (idx)
			{
				case G0_POKRENI:
					group = GROUP_1;
				break;
			}
			break;
		case GROUP_1:
			switch (idx)
			{
				case G1_GLAZBA:
					group = GROUP_2;
					sendAction = 49;
				break;
				case G1_SVJETLO:
					group = GROUP_2;
					sendAction = 51;
				break;
			}
			break;
		case GROUP_2:
			switch(idx) {
				case G2_UPALI:
						group = GROUP_1;
						//Inicijaliziraj nRF
						nRF24L01_initializeDebug(0, 32, 0); //(Tx mode, payload = 32 bytes, autoAck = disable)
						nRF24L01_clearFlush();
						//sprintf(txBuff, "%c", &idx);
						sendAction += 4;
	
						//posalji paket
						nRF24L01_writeTxPayload(&sendAction, 32, 1);
		
						//posalji na UART da je poslano
						uartPutString("Poruka poslana ", USART2);
						GPIO_SetBits(GPIOC, GPIO_Pin_8);
						//pricekaj sekundu
						delay_ms(1000);
				break;
				
				case G2_UGASI:
						group = GROUP_1;
						//Inicijaliziraj nRF
						nRF24L01_initializeDebug(0, 32, 0); //(Tx mode, payload = 32 bytes, autoAck = disable)
						nRF24L01_clearFlush();
						//sprintf(txBuff, "%c", &idx);
						sendAction += 5;
	
						//posalji paket
						nRF24L01_writeTxPayload(&sendAction, 32, 1);
		
						//posalji na UART da je poslano
						uartPutString("Poruka poslana ", USART2);
						//pricekaj sekundu
						delay_ms(1000);
						GPIO_ResetBits(GPIOC, GPIO_Pin_8);
				break;
			}
		break;
		}
}

//prekid svake 1ms
void SysTick_Handler(void)
{
	TimingDelay_Decrement();
}

void USART1_IRQHandler(void) {	
		
		if(USART_GetITStatus(USART1, USART_IT_RXNE)) {
			
			char t = USART_ReceiveData(USART1);
			uartPutString(&t, USART1);
			
			USART_ClearITPendingBit(USART1, USART_IT_RXNE);
		}
	}

