#include "WTV020SD.h"
#include "stm32f0xx_gpio.h"
#include "main.h"

//
//Ports and pins WTV20SD is connected to
//
GPIO_TypeDef * WTV20SD_port = GPIOA;
const unsigned int WTV20SD_resetPin = GPIO_Pin_2;
const unsigned int WTV20SD_clockPin = GPIO_Pin_0;
const unsigned int WTV20SD_dataPin = GPIO_Pin_1;
const unsigned int WTV20SD_busyPin = GPIO_Pin_3;
uint8_t WTV20SD_busyPinState = Bit_SET;

//
//Initialise MCU pins: CLK(PA0), DI(PA1), RST(PA2), BUSY (PA3);
//
void WTV20SD_init(void) {
	
	GPIO_InitTypeDef GPIO_wtvStruct;
	uint32_t WTV20SD_periphClock;
	
	if(WTV20SD_port == GPIOA)
		WTV20SD_periphClock = RCC_AHBPeriph_GPIOA;
	else if (WTV20SD_port == GPIOB)
		WTV20SD_periphClock = RCC_AHBPeriph_GPIOB;
	else if (WTV20SD_port == GPIOC)
		WTV20SD_periphClock = RCC_AHBPeriph_GPIOC;
	else if (WTV20SD_port == GPIOD)
		WTV20SD_periphClock = RCC_AHBPeriph_GPIOD;
	else if (WTV20SD_port == GPIOF)
		WTV20SD_periphClock = RCC_AHBPeriph_GPIOF;
	
	RCC_AHBPeriphClockCmd(WTV20SD_periphClock, ENABLE);
	
	GPIO_StructInit(&GPIO_wtvStruct);
	GPIO_wtvStruct.GPIO_Pin = WTV20SD_clockPin | WTV20SD_dataPin | WTV20SD_resetPin;
	GPIO_wtvStruct.GPIO_Mode = GPIO_Mode_OUT;
	GPIO_wtvStruct.GPIO_OType = GPIO_OType_PP;
	GPIO_wtvStruct.GPIO_PuPd = GPIO_PuPd_NOPULL;
	GPIO_wtvStruct.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(WTV20SD_port, &GPIO_wtvStruct);
	
	GPIO_wtvStruct.GPIO_Pin = WTV20SD_busyPin;
	GPIO_wtvStruct.GPIO_Mode = GPIO_Mode_IN;
	GPIO_Init(WTV20SD_port, &GPIO_wtvStruct);
	
	}

//
//Reset WTV20SD
//
void WTV20SD_reset(void) {
	
	GPIO_WriteBit(WTV20SD_port, WTV20SD_clockPin, Bit_RESET);
	GPIO_WriteBit(WTV20SD_port, WTV20SD_resetPin, Bit_SET);
	//Reset pulse
	GPIO_WriteBit(WTV20SD_port, WTV20SD_resetPin, Bit_RESET);
	delay_10us(500); //5ms
	GPIO_WriteBit(WTV20SD_port, WTV20SD_resetPin, Bit_SET);
	//Reset idle to start bit
	GPIO_WriteBit(WTV20SD_port, WTV20SD_clockPin, Bit_SET);
	delay_10us(30000); //300ms
	
	}

//
//Play voice - syncronized
//
void WTV20SD_playVoice(int voiceNumber) {
	
	WTV20SD_sendCommand(voiceNumber);
	delay_10us(10000); //100ms
	WTV20SD_busyPinState = GPIO_ReadInputDataBit(WTV20SD_port, WTV20SD_busyPin);
	while(!WTV20SD_busyPinState) {
		WTV20SD_busyPinState = GPIO_ReadInputDataBit(WTV20SD_port, WTV20SD_busyPin);
		}
	}

//
//Play voice - ansyncronized
//
void WTV20SD_asyncPlayVoice(int voiceNumber) {
	WTV20SD_sendCommand(voiceNumber);
	}

//
//Stop voice
//
void WTV20SD_stopVoice(void) {
	WTV20SD_sendCommand(WTV20SD_STOP);
	}

//
//Pause voice
//
void WTV20SD_pauseVoice(void) {
	WTV20SD_sendCommand(WTV20SD_PLAY_PAUSE);
	}

//
//Mute - minimum volume
//
void WTV20SD_mute(void) {
	WTV20SD_sendCommand(WTV20SD_VOLUME_MIN);
	}

//
//Unmute - maximum volume
//
void WTV20SD_unmute(void) {
	WTV20SD_sendCommand(WTV20SD_VOLUME_MAX);
	}

//
//Adjust volume
//
void WTV20SD_volume(int level) {
	if (level > 0 & level < 8) {
		unsigned int vLevel = WTV20SD_VOLUME_MIN + level;
		WTV20SD_sendCommand(vLevel);
		}
	}

//
//Send command function
//
void WTV20SD_sendCommand(unsigned int command) {
	
	unsigned int mask = 0x8000;
	
	//Start bit Low level pulse
	GPIO_WriteBit(WTV20SD_port, WTV20SD_clockPin, Bit_RESET);
	delay_10us(200); //2ms
	
	for (mask = 0x8000; mask > 0; mask >>=1) {
		//Clock low level pulse
		GPIO_WriteBit(WTV20SD_port, WTV20SD_clockPin, Bit_RESET);
		delay_10us(5); //50us - mozda ne treba
		
		//Write data setup - ovo je sa MSB first, mozda LSB first?
		if (command & mask) 
			GPIO_WriteBit(WTV20SD_port, WTV20SD_dataPin, Bit_SET);
		else
			GPIO_WriteBit(WTV20SD_port, WTV20SD_dataPin, Bit_RESET);
			
		//Write data hold
		delay_10us(5); //50us
		//Clock high level pulse
		GPIO_WriteBit(WTV20SD_port, WTV20SD_clockPin, Bit_SET);
		delay_10us(10); //100us
		if (mask > 0x0001)
			delay_10us(200); //Stop bit high level pulse; 2ms
		}
	//Busy active high from last data bit latch
	delay_10us(2000); //20ms
	}
