#include "stm32f0xx_gpio.h"
#include "uart.h"
#include "spi.h"
#include "stdio.h"
#include "main.h"
#include "nRF24L01.h"
#include "nRF24L01Defs.h"
#include "nRF24L01_test.h"
#include "WTV020SD.h"

#define MAXSTRLEN 32
#define MAXSENZLEN 8

char rcv_data [MAXSTRLEN + 1];
static uint32_t TimingDelay;

GPIO_InitTypeDef zelena;

//Delay u destcima mikrosekundi delay(1) = 10us
void delay_10us(uint32_t nTime)
{ 
  TimingDelay = nTime;

  while(TimingDelay != 0);
}

void TimingDelay_Decrement(void)
{
	if (TimingDelay != 0x00)
  { 
    TimingDelay--;
  }
}

int main() {
	
	//Postavi SysTick da broji po 10us, racuna se prema formuli:
	//Reload Value = SysTick Counter Clock (Hz) x  Desired Time base (s)
	if (SysTick_Config(SystemCoreClock / 100000))
  { 
    /* Capture error */ 
    while (1);
  }
	NVIC_SetPriority(SysTick_IRQn, 0);
	
	//Crvena ledica
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOC, ENABLE);
	GPIO_StructInit(&zelena);
	zelena.GPIO_Pin = GPIO_Pin_7;
	zelena.GPIO_Mode = GPIO_Mode_OUT;
	zelena.GPIO_Speed = GPIO_Speed_50MHz;
	zelena.GPIO_OType = GPIO_OType_PP;
	GPIO_Init(GPIOC, &zelena);
	
	//Inicijalizacija USART1
	uart_open(USART1, 9600);
	uart_put_string("Inicijalizacija je uspjela\n", USART1);
	
	//Inicijalizacija SPI1 za nRF24L01
	spi_nrfInit();
	
	WTV20SD_init();
	WTV20SD_reset();
	WTV20SD_unmute();
	WTV20SD_volume(4);
	delay_10us(20000);	
	
	testUnidirectionalMessagingRx();
	
	while(1) {

		}
	}

//prekid svake 10us
void SysTick_Handler(void)
{
	TimingDelay_Decrement();
}

void USART1_IRQHandler(void) {	
		
		if(USART_GetITStatus(USART1, USART_IT_RXNE)) {
			
			//static uint8_t cnt = 0;
			char t = USART_ReceiveData(USART1);
			
			//testUnidirectionalMessagingTxMore();
			
			if(t == 'F' && GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_9)) {
				delay_10us(100);
				GPIO_ResetBits(GPIOC, GPIO_Pin_9);
				uart_put_string("Gasi! ", USART1);
			}
			else if (t == 'O' && !GPIO_ReadInputDataBit(GPIOC, GPIO_Pin_9)) {
				delay_10us(100);
				GPIO_SetBits(GPIOC, GPIO_Pin_9);
				uart_put_string("Pali! ", USART1);
			}
			
			USART_ClearITPendingBit(USART1, USART_IT_RXNE);
		}
	}
