#include "nRF24L01Defs.h"

void nRF24L01_CSNSelect(void);
void nRF24L01_CSNDeselect(void);
void nRF24L01_CEHigh(void);
void nRF24L01_CELow(void);
unsigned char SPI_readWriteByte(unsigned char byte);
//
// Initialization functions
//
void nRF24L01_initialize(unsigned char config,
						unsigned char opt_rx_active_mode,
						unsigned char en_aa,
						unsigned char en_rxaddr,
						unsigned char setup_aw,
						unsigned char setup_retr,
						unsigned char rf_ch,
						unsigned char rf_setup,
						unsigned char *rx_addr_p0,
						unsigned char *rx_addr_p1,
						unsigned char rx_addr_p2,
						unsigned char rx_addr_p3,
						unsigned char rx_addr_p4,
						unsigned char rx_addr_p5,
						unsigned char *tx_addr,
						unsigned char rx_pw_p0,
						unsigned char rx_pw_p1,
						unsigned char rx_pw_p2,
						unsigned char rx_pw_p3,
						unsigned char rx_pw_p4,
						unsigned char rx_pw_p5);
void nRF24L01_initializeDebug(bool rx, unsigned char p0PayloadWidth, bool enableAutoAck);
void nRF24L01_initializeDebugLite(bool rx, unsigned char p0PayloadWidth);

//
// Power-up, Power-down functions
//
void nRF24L01_powerUp(bool rxActiveMode);
void nRF24L01_powerUpParam(bool rxActiveMode, unsigned char config);
void nRF24L01_powerDown(void);
void nRF24L01_powerDownParam(unsigned char config);

//
// RX/TX functions
//
void nRF24L01_setAsRx(bool rxActiveMode);
void nRF24L01_setAsRxParam(bool rxActiveMode, unsigned char config);
void nRF24L01_rxStandbyToActive(void);
void nRF24L01_rxActiveToStandby(void);
void nRF24L01_setAsTx(void);
void nRF24L01_setAsTxParam(unsigned char config);

//
// SPI command defined by the spec
// for regnumber values, see section above titled "register definitions"
// all functions return the STATUS register
//
unsigned char nRF24L01_writeRegister(unsigned char regnumber, unsigned char * data, unsigned int len);
unsigned char nRF24L01_readRegister(unsigned char regNumber, unsigned char * data, unsigned int len);
unsigned char nRF24L01_writeTxPayload(unsigned char *data, unsigned int len, bool transmit);
unsigned char nRF24L01_readRxPayload(unsigned char *data, unsigned int len);
unsigned char nRF24L01_flushTx(void);
unsigned char nRF24L01_flushRx(void);
unsigned char nRF24L01_reuseTxPl(void);
unsigned char nRF24L01_nop(void);

//
// IO interface-related functions (together with first 4 functions in this file
//
void nRF24L01_transmit(void);
bool nRF24L01_CEPinActive(void);
bool nRF24L01_CSNPinActive(void);

//
// Register-oriented get/set functions for commonly-used registers during operations
//
void nRF24L01_setRxAddr(unsigned char *address, unsigned int len, unsigned char rxPipeNum);
void nRF24L01_setTxAddr(unsigned char *address, unsigned int len);
void nRF24L01_setRxPw(unsigned char payloadWidth, unsigned char rxPipeNum);
unsigned char nRF24L01_getRxPw(unsigned char rxPipeNum);
unsigned char nRF24L01_getConfig(void);
void nRF24L01_setConfig(unsigned char config);
unsigned char nRF24L01_getRfCh(void);
void nRF24L01_setRfCh(unsigned char channel);
unsigned char nRF24L01_getObserveTx(void);
unsigned char nRF24L01_getStatus(void);
unsigned char nRF24L01_getFifoStatus(void);

//
// Auto-ack and Pipe-related functions
//
bool nRF24L01_aaEnabled(unsigned char rxPipeNum);
void nRF24L01_aaEnable(unsigned char rxPipeNum);
void nRF24L01_aaDisable(unsigned char rxPipeNum);
bool nRF24L01_rxPipeEnabled(unsigned char rxPipeNum);
void nRF24L01_rxPipeEnable(unsigned char rxPipeNum);
void nRF24L01_rxPipeDisable(unsigned char rxPipeNum);
unsigned char nRF24L01_getPlosCnt(void);
void nRF24L01_clearPlosCnt(void);
void nRF24L01_clearPlosCntParam(unsigned char rfCh);
unsigned char nRF24L01_getArcCnt(void);

//
// Utility functions
//
bool nRF24L01_cdActive(void);
void nRF24L01_clearFlush(void);
unsigned char nRF24L01_getRxPipe(void);
unsigned char nRF24L01_getRxPipeFromStatus(unsigned char status);
void nRF24L01_getAllRegisters(unsigned char *data);

//
// FIFO-STATUS check functions
//
bool nRF24L01_fifoTxReuse(void);
bool nRF24L01_fifoTxFull(void);
bool nRF24L01_fifoTxEmpty(void);
bool nRF24L01_fifoRxFull(void);
bool nRF24L01_fifoRxEmpty(void);

//
// Interrupt check/clear functions
//
bool nRF24L01_irqPinActive(void);
bool nRF24L01_irqRxDrActive(void);
bool nRF24L01_irqTxDsActive(void);
bool nRF24L01_irqMaxRtActive(void);
void nRF24L01_irqClearAll(void);
void nRF24L01_irqClearRxDr(void);
void nRF24L01_irqClearTxDs(void);
void nRF24L01_irqClearMaxRt(void);

//
//Low-level functions for library use only
//
unsigned char nRF24L01_executeCommand(unsigned char instruction, unsigned char *data, unsigned int len, bool copydata);
void nRF24L01_SPISendRead(unsigned char *data, unsigned int len, bool copydata);
